@@ -5,6 +5,7 @@
 #include <xf86drm.h>
 #include <xf86drmMode.h>
 #include <sys/mman.h>
+#include <sstream>
 #include "Shared.hpp"
 #include "aquamarine/output/Output.hpp"
 
@@ -99,8 +100,19 @@ void Aquamarine::CDRMAtomicRequest::addConnector(Hyprutils::Memory::CSharedPoint
     TRACE(backend->log(AQ_LOG_TRACE, std::format("atomic addConnector values: CRTC {}, mode {}", enable ? connector->crtc->id : 0, data.atomic.modeBlob)));
 
     conn = connector;
+    if (enable) {
+        drmModeModeInfo* currentMode = connector->getCurrentMode();
+        bool             modeDiffers = true;
+        if (currentMode) {
+            modeDiffers = memcmp(currentMode, &data.modeInfo, sizeof(drmModeModeInfo)) != 0;
+            free(currentMode);
+        }
 
-    addConnectorModeset(connector, data);
+        if (modeDiffers)
+            addConnectorModeset(connector, data);
+    } else 
+        addConnectorModeset(connector, data);
+    
     addConnectorCursor(connector, data);
 
     add(connector->id, connector->props.crtc_id, enable ? connector->crtc->id : 0);
@@ -180,7 +192,7 @@ void Aquamarine::CDRMAtomicRequest::addConnectorModeset(Hyprutils::Memory::CShar
     const auto& STATE  = connector->output->state->state();
     const bool  enable = STATE.enabled && data.mainFB;
 
-    add(connector->crtc->id, connector->crtc->props.mode_id, data.atomic.modeBlob);
+    add(connector->crtc->id, connector->crtc->props.mode_id, enable ? data.atomic.modeBlob : 0);
     data.atomic.blobbed = true;
 
     if (!enable)
@@ -223,20 +235,20 @@ void Aquamarine::CDRMAtomicRequest::addConnectorCursor(Hyprutils::Memory::CShare
 
 bool Aquamarine::CDRMAtomicRequest::commit(uint32_t flagssss) {
     static auto flagsToStr = [](uint32_t flags) {
-        std::string result;
+        std::ostringstream result;
         if (flags & DRM_MODE_ATOMIC_ALLOW_MODESET)
-            result += "ATOMIC_ALLOW_MODESET ";
+            result << "ATOMIC_ALLOW_MODESET ";
         if (flags & DRM_MODE_ATOMIC_NONBLOCK)
-            result += "ATOMIC_NONBLOCK ";
+            result << "ATOMIC_NONBLOCK ";
         if (flags & DRM_MODE_ATOMIC_TEST_ONLY)
-            result += "ATOMIC_TEST_ONLY ";
+            result << "ATOMIC_TEST_ONLY ";
         if (flags & DRM_MODE_PAGE_FLIP_EVENT)
-            result += "PAGE_FLIP_EVENT ";
+            result << "PAGE_FLIP_EVENT ";
         if (flags & DRM_MODE_PAGE_FLIP_ASYNC)
-            result += "PAGE_FLIP_ASYNC ";
+            result << "PAGE_FLIP_ASYNC ";
         if (flags & (~DRM_MODE_ATOMIC_FLAGS))
-            result += " + invalid...";
-        return result;
+            result << " + invalid...";
+        return result.str();
     };
 
     if (failed) {
